<?php
namespace SCM\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Error;
use SCM\Traits\TResponse;

class Solicitacao extends \Zeedhi\Framework\Controller\Simple {
    use TResponse;

	protected $solicitacao;
	protected $environment;

	public function __construct(\SCM\Services\Solicitacao $solicitacao, $environment) {
        $this->solicitacao = $solicitacao;
        $this->environment = $environment;
    }

	public function retornaDatasEntregaSolicitacao(Request $request, Response $response) {
        try {
        	$filter = $request->getFilterCriteria()->getConditions();
        	
            // $nrorg = $filter[0]['value'];
            $nrorg = $this->environment->getNrorg();
            $cdfilial = $filter[1]['value'];
            $dataInicial = $filter[2]['value'];
            $dataFinal = $filter[3]['value'];
            $periodo = $filter[4]['value'];

            $datasEntraga = $this->solicitacao->retornaDatasEntregaSolicitacao($nrorg, $cdfilial, $dataInicial, $dataFinal);

			$response->addDataSet(new DataSet('DatasEntregaSolicitacao', $datasEntraga));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function retornaProdutosSolicitacao(Request $request, Response $response) {
        try {
        // 	$filter = $request->getFilterCriteria()->getConditions();
            $params = $request->getParameters();
            
            // $nrorg = $filter[0]['value'];
            $nrorg = $this->environment->getNrorg();
            $filiais = $params['CDFILIAL'];
            $dataInicial = $params['DTINICIAL'];
            $dataFinal = $params['DTFINAL'];
       
            $produtos = $this->solicitacao->retornaProdutosSolicitacao($nrorg, $filiais, $dataInicial, $dataFinal);    
           
            $this->sendResponse($produtos);
            // $response->addDataSet(new DataSet('ProdutosSolicitacao', $produtos));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function retornaSolicitacoes(Request $request, Response $response) {
        try {
        	$filter = $request->getFilterCriteria()->getConditions();
        	
            // $nrorg = $filter[0]['value'];
            $nrorg = $this->environment->getNrorg();
            $cdfilial = $filter[1]['value'];
            $dataInicial = $filter[2]['value'];
            $dataFinal = $filter[3]['value'];
            $periodo = $filter[4]['value'];

            $solicitacoes = $this->solicitacao->retornaSolicitacoes($nrorg, $cdfilial, $dataInicial, $dataFinal);

			$response->addDataSet(new DataSet('Solicitacao', $solicitacoes));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function retornaProximoCodigoSolicitacao(Request $request, Response $response) {
        try {
        	$filter = $request->getFilterCriteria()->getConditions();
        	
            // $nrorg = $filter[0]['value'];
            $nrorg = $this->environment->getNrorg();
            $cdfilial = $filter[1]['value'];

            $codigo = $this->solicitacao->retornaProximoCodigoSolicitacao($nrorg, $cdfilial);

			$response->addDataSet(new DataSet('ProximoCodigoSolicitacao', $codigo));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function novaSolicitacao(Request\Dataset $request, Response $response) {
        
        try {

            $rows = $request->getDataSet()->getRows();

            $this->solicitacao->novaSolicitacao($rows);

        } catch (\Exception $e) {

            $response->setError(new Error($e->getMessage(), $e->getCode()));

        }
    }
    
    public function salvaProduto(Request\Dataset $request, Response $response) {
        try {
            $rows = $request->getDataSet()->getRows();
            $nrorg = $this->environment->getNrorg();
            $cdoperador = $this->environment->getCdOperador();
            $produtosJaAprovados = $this->solicitacao->salvaProduto($rows, $nrorg, $cdoperador);

            // $response->addDataSet(new Dataset('ProdutosAprovados', $produtosJaAprovados));
            $this->sendResponse($produtosJaAprovados);
        } catch (\Exception $e) {

            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function salvaProdutoEditado(Request\Dataset $request, Response $response) {
       
        try {
            $rows = $request->getDataSet()->getRows();

            $produtosJaAprovados = $this->solicitacao->salvaProdutoEditado($rows);
            
            if (!empty($produtoJaAprovado)) {
                $errorStr = "ERROR_CHANGE_QUANTITY_EXPORT";
                // Os seguintes produtos não tiveram sua quantidade ajustada, pois já estavam aprovados no momento da exportação: 
                foreach($produtoJaAprovado as $prod) {
                    $errorStr .= $prod['CDPRODUTO']." - ".$prod['NMPRODUTO'];
                }
                throw new \Exception($errorStr);
            }

            // $response->addDataSet(new Dataset('ProdutosAprovados', $produtosJaAprovados));
            $this->sendResponse($produtosJaAprovados);
        } catch (\Exception $e) {

            $response->setError(new Error($e->getMessage(), $e->getCode()));
        }
    }

    public function excluirProdutos(Request\Dataset $request, Response $response) {
        try {
            $rows = $request->getDataSet()->getRows();
            $nrorg = $this->environment->getNrorg();
            $produtosJaAprovados = array();
            
            // $produtoJaAprovado = $this->solicitacao->excluirProdutos($rows, $nrorg);
            $produtoJaAprovado = array(array('CDPRODUTO'=>'teste','NMPRODUTO'=>'teste'));

            if (!empty($produtoJaAprovado)) {
                $errorStr = "ERROR_DELETE_PRODUCT_EXPORT";
                // Os seguintes produtos não foram excluídos, pois já estavam aprovados no momento da exportação: 
                foreach($produtoJaAprovado as $prod) {
                    $errorStr .= $prod['CDPRODUTO']." - ".$prod['NMPRODUTO'];
                }
                throw new \Exception($errorStr);
            }

            $this->sendResponse($produtosJaAprovados);
            // $response->addDataSet(new Dataset('ProdutosAprovados', $produtosJaAprovados));

        } catch (\Exception $e) {

            $response->setError(new Error($e->getMessage(), $e->getCode()));

        }
    }
}