<?php

namespace SCM\Services;

class Solicitacao {

    private $entityManager;
    private $parametros;
    private $newCode;
    private $novoCodigo = array();
    private $environment;
    private $movopsol;
    
    public function __construct(\Doctrine\ORM\EntityManager $entityManager, \SCM\Services\Parametros $parametros, $newCode, $environment,
        \SCM\Services\Movopsol $movopsol) {
        $this->entityManager = $entityManager;
        $this->parametros = $parametros;
        $this->newCode = $newCode;
        $this->environment = $environment;
        $this->movopsol = $movopsol;
    }

    public function retornaDatasEntregaSolicitacao($nrorg, $cdfilial, $dataInicial, $dataFinal) {
        $datasEntrega = $this->entityManager->getRepository('SCM\Model\Solicitacao')->retornaDatasEntregaSolicitacao($nrorg, $cdfilial, $dataInicial, $dataFinal);
        $rownum = 1;
        $novasDatasEntrega = array();

        foreach ($datasEntrega as $data) {
            
            $data['ROWNUM'] = $rownum;
            $rownum++;

            $novasDatasEntrega[] = $data;

        }

        return $novasDatasEntrega;
    }

    public function retornaProdutosSolicitacao($nrorg, $cdfilial, $dataInicial, $dataFinal) {
        return $this->entityManager->getRepository('SCM\Model\Solicitacao')->retornaProdutosSolicitacao($nrorg, $cdfilial, $dataInicial, $dataFinal);
    }

    public function retornaSolicitacoes($nrorg, $cdfilial, $dataInicial, $dataFinal) {
        return $this->entityManager->getRepository('SCM\Model\Solicitacao')->retornaSolicitacoes($nrorg, $cdfilial, $dataInicial, $dataFinal);
    }

    public function retornaProximoCodigoSolicitacao($nrorg, $cdfilial) {
        return $this->entityManager->getRepository('SCM\Model\Solicitacao')->retornaProximoCodigoSolicitacao($nrorg, $cdfilial);
    }

    /**
     * @param Array $solicitacoes array de solicitações, contendo array de produtos em cada solicitação
     */
    public function novaSolicitacao($solicitacoes) {

        $solicita = new \SCM\Factory\Solicita();

        $solicitacoesSalvas = array();

        $this->entityManager->getConnection()->beginTransaction();

        foreach ($solicitacoes as $solicitacao) {
            $solicitacao['NRORG'] = $this->environment->getNrorg();
            $solicitacao['CDOPERADOR'] = $this->environment->getCdOperador();
            $novoCodigoSolicitacao = $this->retornaCodigoSolicitacao($solicitacao);

            $novaSolicitacao = $solicita->criarSolicitacao($solicitacao, $novoCodigoSolicitacao);
            
            if ($novaSolicitacao->getQtitemsolic() > 0) {

                $this->entityManager->persist($novaSolicitacao);
            }

            $solicitacao['NRSOLICMP'] = $novoCodigoSolicitacao;
            $solicitacao['PRODUTOS'] = $this->atualizaProdutosSolicitacao($novoCodigoSolicitacao, $solicitacao['PRODUTOS']);
            $solicitacao['LOG'] = $this->atualizaLogsSolicitacao($novoCodigoSolicitacao, $solicitacao['LOG']);

            $solicitacoesSalvas[] = $solicitacao;

        }

        $this->entityManager->flush();
        $this->entityManager->getConnection()->commit();

        $this->salvaProdutosDasSolicitacoes($solicitacoesSalvas);
        $this->salvaMovopsolDasSolicitacoes($solicitacoesSalvas);
        $this->salvaMovsolextDasSolicitacoes($solicitacoesSalvas);

        $this->novoCodigo = array();

    }

    /**
     * @param Array $solicitacoes array de solicitações, contendo array de produtos em cada solicitação
     */
    public function salvaProdutosDasSolicitacoes($solicitacoes) {

        $itemsoli = new \SCM\Factory\Itemsoli();

        $this->entityManager->getConnection()->beginTransaction();    

        foreach ($solicitacoes as $solicitacao) {

            foreach ($solicitacao['PRODUTOS'] as $produto) {
                $produto['NRORG'] = $this->environment->getNrorg();
                $produto['CDOPERADOR'] = $this->environment->getCdOperador();
                
                $params = array(
                    'cdfilsoli' => $produto['CDFILSOLI'],
                    'cdfillan' => $produto['CDFILLAN'],
                    'nrsolicmp' => $produto['NRSOLICMP'],
                    'cdproduto' => $produto['CDPRODUTO'],
                    'dtprogentr' => $this->toDate($produto['DTPROGENTR']),
                );
                
                $produtoExiste = $this->entityManager->getRepository('SCM\Model\Itemsoli')->findBy($params);

                if(empty($produtoExiste)) {
                    $produto['CDFILIOPELOG'] = $this->entityManager->getRepository('SCM\Model\Solicitacao')->retornaCdfiliopelog($produto, $solicitacao['CDFILCENT']);
                    $novoProduto = $itemsoli->criaProduto($produto);
                    $this->entityManager->persist($novoProduto);

                } else {
                    
                    $produtoExiste[0]->setQtsoliprod($produto['QTAJUSTADA']);
                    $this->entityManager->persist($produtoExiste[0]);
                }

            }

        }

        $this->entityManager->flush();
        $this->entityManager->getConnection()->commit();

    }

    /**
     * @param Array $solicitacoes array de solicitações, contendo array de produtos em cada solicitação
     */
    public function salvaMovopsolDasSolicitacoes($solicitacoes) {

        $movopsol = new \SCM\Factory\Movopsol();

        $this->entityManager->getConnection()->beginTransaction();

        foreach ($solicitacoes as $solicitacao) {

            foreach ($solicitacao['LOG'] as $log) {
                $log['CDOPERADOR'] = $this->environment->getCdOperador();
                $log['NRORG'] = $this->environment->getNrorg();
                $novoLog = $movopsol->criaMovopsol($log);
                $this->entityManager->persist($novoLog);

            }

        }

        $this->entityManager->flush();
        $this->entityManager->getConnection()->commit();

    }

    /**
     * @param Array $solicitacoes array de solicitações, contendo array de produtos em cada solicitação
     */
    public function salvaMovsolextDasSolicitacoes($solicitacoes) {

        $movsolext = new \SCM\Factory\Movsolext();

        $this->entityManager->getConnection()->beginTransaction();

        foreach ($solicitacoes as $solicitacao) {
            if ($solicitacao["IDSOLEXTRA"] == 'S') {
                if (isset($solicitacao["LOG"]) && !empty($solicitacao["LOG"])) {
                    $solicitacao["LOG"][0]["NRORG"] = $this->environment->getNrorg();
                    $solicitacao["LOG"][0]["CDOPERADOR"] = $this->environment->getCdOperador();
                    $novoLog = $movsolext->criaMovsolext($solicitacao["LOG"][0]);
                    $this->entityManager->persist($novoLog);
                }
            }
        }

        $this->entityManager->flush();
        $this->entityManager->getConnection()->commit();

    }
    
    public function salvaProduto($produtos, $nrorg, $cdoperador) {
        $itemsoli = new \SCM\Factory\Itemsoli();
        $this->entityManager->getConnection()->beginTransaction();
        
        foreach ($produtos as $produto) {
            $produto['NRORG'] = $nrorg;
            $produto['CDOPERADOR'] = $cdoperador;
            
            $params = array(
                'cdfilsoli'  => $produto['CDFILSOLI'],
                'cdfillan'   => $produto['CDFILLAN'],
                'nrsolicmp'  => $produto['NRSOLICMP'],
                'cdproduto'  => $produto['CDPRODUTO'],
                'dtprogentr' => $produto['DTPROGENTR'],
            );
                
            $produtoExiste = $this->entityManager->getRepository('SCM\Model\Itemsoli')->findBy($params);
            
            if(empty($produtoExiste)) {
                // $produto['NRSOLICMP'] = $novoCodigoSolicitacao;
                $produto['CDFILIOPELOG'] = $this->entityManager->getRepository('SCM\Model\Solicitacao')->retornaCdfiliopelog($produto, $produto['CDFILCENT']);
                $novoProduto = $itemsoli->criaProduto($produto);
                $this->entityManager->persist($novoProduto);
            } else {
                $produtoExiste[0]->setQtsoliprod($produto['QTAJUSTADA']);
                $this->entityManager->persist($produtoExiste[0]);
            }
            
            $this->movopsol->criaMovopsol([$produto['LOG']]);
        }
            
        $this->entityManager->flush();
        $this->entityManager->getConnection()->commit();
    }

    /**
     * @param Array $produtos
     */
    public function salvaProdutoEditado($produtos) {
        
        $itemsoli = new \SCM\Factory\Itemsoli();

        $this->entityManager->getConnection()->beginTransaction();
        $produtosJaAprovados = array();

        foreach ($produtos as $produto) {
            $produto['NRORG'] = $this->environment->getNrorg();
            $produto['CDOPERADOR'] = $this->environment->getCdOperador();
            $params = array(
                'cdfilsoli' => $produto['CDFILSOLI'],
                'cdfillan' => $produto['CDFILLAN'],
                'nrsolicmp' => $produto['NRSOLICMP'],
                'cdproduto' => $produto['CDPRODUTO'],
                'dtprogentr' => $this->toDate($produto['DTPROGENTR']),
            );

            $produtoExiste = $this->entityManager->getRepository('SCM\Model\Itemsoli')->findBy($params);

            if(!empty($produtoExiste)) {
                if($produto['QTAJUSTADA'] <= 0 ) {
                    $this->excluirProdutos($produto);                    
                } else {
                    $parametrosFilial = $this->parametros->retornaParametros($produto['CDFILSOLI'], $produto['NRORG']);
                    if ($parametrosFilial[0]['IDUTAPROVSOL'] == 'S') {
                        if ($parametrosFilial[0]['ALTERAR_PRODUTO_APROVADO'] == 'N' && $produtoExiste[0]->getIdsoliaprov() == "S") {
                            $produtosJaAprovados[] = $this->retornaProdutoJaAprovado($produtoExiste[0], $produto);
                        } else {
                            $produtoExiste[0]->setQtsoliprod($produto['QTAJUSTADA']);
                            $produtoExiste[0]->setIdsoliaprov($produto['IDSOLIAPROV']);
                            $this->entityManager->persist($produtoExiste[0]);
                            $this->movopsol->criaMovopsol([$produto['LOG']]);
                        }
                    } else {
                        $produtoExiste[0]->setQtsoliprod($produto['QTAJUSTADA']);
                        $produtoExiste[0]->setIdsoliaprov($produto['IDSOLIAPROV']);
                        $this->entityManager->persist($produtoExiste[0]);
                        $this->movopsol->criaMovopsol([$produto['LOG']]);
                    }
                }
            }
        }

        $this->entityManager->flush();
        $this->entityManager->getConnection()->commit();

        return $produtosJaAprovados;
    }

    private function retornaProdutoJaAprovado($produto, $produtoAntigo) {
        return array(
            "CDFILSOLI" => $produto->getCdfilsoli(),
            "CDFILLAN" => $produto->getCdfillan(),
            "NRSOLICMP" => $produto->getNrsolicmp(),
            "CDPRODUTO" => $produto->getCdproduto(),
            "NMPRODUTO" => $produtoAntigo['NMPRODUTO'],
            "DTPROGENTR" => \DateTime::createFromFormat('Y-m-d H:i:s', $produto->getDtprogentr())->format('d/m/Y'),
            "QTSOLIPROD" => $produto->getQtsoliprod(),
            "DTUTILIZA" => \DateTime::createFromFormat('Y-m-d H:i:s', $produto->getDtutiliza())->format('d/m/Y'),
            "IDSOLIAPROV" => $produto->getIdsoliaprov(),
            "QTVOLUITEMSO" => $produto->getQtvoluitemso(),
            "NRORG" => $produto->getNrorg(),
            "CDFILIOPELOG" => $produto->getCdfiliopelog(),
            "QTAJUSTADA" => $produtoAntigo['QTAJUSTADA'],
            "DSJUSTOPER" => $produtoAntigo['DSJUSTOPER'],
        );
    }

    private function atualizaProdutosSolicitacao($codigoSolicitacao, $produtos) {
        
        for ($i = 0; $i < sizeof($produtos); $i++) {
            
            $produtos[$i]['NRSOLICMP'] = $codigoSolicitacao;
        }

        return $produtos;
    }

    private function atualizaLogsSolicitacao($codigoSolicitacao, $logs) {
        
        for ($i = 0; $i < sizeof($logs); $i++) {
            
            $logs[$i]['NRSOLIMOV'] = $codigoSolicitacao;
        }

        return $logs;
    }

    private function retornaCodigoSolicitacao($solicitacao) {
        if(array_key_exists($solicitacao['CDFILSOLI'], $this->novoCodigo)) {
            
            // Rotina alterada para que o novo código gerado seja igual ao código gerado no delphi
            
            $novoCodigoSolicitacao = $this->newCode->getCodeByTableName('SOLICOMP'.$solicitacao['CDFILSOLI'].$solicitacao['CDFILLANCA'], $solicitacao['NRORG']);

        } else {

            // Rotina alterada para que o novo código gerado seja igual ao código gerado no delphi
            
            $novoCodigoSolicitacao = $this->newCode->getCodeByTableName('SOLICOMP'.$solicitacao['CDFILSOLI'].$solicitacao['CDFILLANCA'], $solicitacao['NRORG']);
            $this->setNovoCodigo($novoCodigoSolicitacao, $solicitacao['CDFILSOLI']);
        }
        return $novoCodigoSolicitacao;
    }

    private function setNovoCodigo($novoCodigoSolicitacao, $cdfilsoli) {
        $this->novoCodigo[$cdfilsoli] = $novoCodigoSolicitacao;
    }

    private function retornaNovoCodigoSolicitacao($cdfilsoli) {

        $codigoAtual = intval($this->novoCodigo[$cdfilsoli]);
        $codigoAtual++;
        $codigoAtual = '0000000000'.$codigoAtual;
        $codigoAtual = substr($codigoAtual, -10);

        $this->novoCodigo[$cdfilsoli] = $codigoAtual;

        return $codigoAtual;
    }

    /**
     * @param Array $row
     */
    public function excluirProdutos($rows, $nrorg) {
        $produtosJaAprovados = array();
        $this->entityManager->getConnection()->beginTransaction();
        foreach ($rows as $row) {
            $produtoJaAprovado = array();
            $removed = false;
            $params = array(
                'cdfilsoli' => $row['CDFILSOLI'],
                'cdfillan' => $row['CDFILLAN'],
                'nrsolicmp' => $row['NRSOLICMP'],
                'cdproduto' => $row['CDPRODUTO'],
                'dtprogentr' => $row['DTPROGENTR'],
            );
            
            $produtoExiste = $this->entityManager->getRepository('SCM\Model\Itemsoli')->findBy($params);
            
            $todosProdutosSolicitacao = $this->entityManager->getRepository('SCM\Model\Itemsoli')->findBy(array(
                'cdfilsoli' => $row['CDFILSOLI'],
                'cdfillan' => $row['CDFILLAN'],
                'nrsolicmp' => $row['NRSOLICMP'],
            ));
            
            if(!empty($produtoExiste)) {
                $parametrosFilial = $this->parametros->retornaParametros($row['CDFILSOLI'], $nrorg);
                $row['QTAJUSTADA'] = 0;
                $row['DSJUSTOPER'] = 'EXCLUSÃO';
                if ($parametrosFilial[0]['IDUTAPROVSOL'] == 'S') {
                    if ($produtoExiste[0]->getIdsoliaprov() == "S") {
                        $produtoJaAprovado = $this->retornaProdutoJaAprovado($produtoExiste[0], $row);
                    } else {
                        $produtoExiste[0]->setQtsoliprod($row['QTAJUSTADA']);
                        $produtoExiste[0]->setIdsoliaprov($row['IDSOLIAPROV']);
                        $removed = true;
                        $this->entityManager->remove($produtoExiste[0]);
                    }
                } else {
                    $produtoExiste[0]->setQtsoliprod($row['QTAJUSTADA']);
                    $produtoExiste[0]->setIdsoliaprov($row['IDSOLIAPROV']);
                    $removed = true;
                    $this->entityManager->remove($produtoExiste[0]);
                }
            }
            
            if ($removed) {
                $this->movopsol->criaMovopsol([$row['LOG']]);
            }
            
            if(sizeof($todosProdutosSolicitacao) == 1 && $removed) {
                $solicitacao = $this->entityManager->getRepository('SCM\Model\Solicita')->findBy(array(
                    'cdfilsoli' => $row['CDFILSOLI'],
                    'cdfillanca' => $row['CDFILLAN'],
                    'nrsolicmp' => $row['NRSOLICMP'],
                ));
    
                if (!empty($solicitacao)) {
                    $this->entityManager->remove($solicitacao[0]);
                    
                }
            }
            if (!empty($produtoJaAprovado)) $produtosJaAprovados[] = $produtoJaAprovado;
        }
        
        $this->entityManager->flush();
        $this->entityManager->getConnection()->commit();
        
        return $produtosJaAprovados;
    }

    private function toDate($stringDate) {
        // $explodeDate = explode("/", $stringDate);
        // $date = $explodeDate[2].'-'.$explodeDate[1].'-'.$explodeDate[0];
        return $stringDate;
    }
}