import {
  Loader, Metadata, IEventParam, I18n, dayjs as moment, Config, Router,
} from '@zeedhi/core';
import {
  Card, IterableComponentRender, Tabs, Tab, LoadingService, Form, Text, Button, Select, DialogService,
} from '@zeedhi/common';
import { BottomSheet } from '@zeedhi/zd-bottom-sheet';
import { ScmMask } from '@/config';

export class AFController {
  private utilities = Loader.getInstance('Utilities');

  private appController = Loader.getInstance('AppController');

  public showAfDetail = false;

  private periodo = {
    DTINICIAL: '',
    DTFINAL: '',
  };

  public get setFilterAFs() {
    if (localStorage.getItem('hasImportedData') === 'true') {
      return {
        CDFILIAL: localStorage.getItem('CDFILIALLOGADA') || '',
        DTPROGENTAF: this.utilities.getAllDatesInRange(this.periodo.DTINICIAL, this.periodo.DTFINAL),
      };
    }
    return {
      CDFILIAL: localStorage.getItem('CDFILIALLOGADA') || '',
      DTINICIAL: this.periodo.DTINICIAL || '',
      DTFINAL: this.periodo.DTFINAL || '',
    };
  }

  get isVisibleDetailList() {
    return !this.showAfDetail;
  }

  public setValueQtde({ component }: IEventParam<Text>) {
    const value = component.name.split('-').reverse()[0];
    // const mask = {
    //   decimalPlates: 5,
    // };
    const mask = Config.masks.productMask as ScmMask;
    const maskNumber = this.utilities.getMaskNumber(value, mask);
    component.text = `${I18n.translate('Qtde')}: ${maskNumber}`;
  }

  public async clickCardAf({ component }: IEventParam<Card>) {
    LoadingService.show(' ');
    const mask = {
      decimalPlates: 5,
    };
    this.showAfDetail = true;
    const tabsAfs = Metadata.getInstance<Tabs>('tabsAF');
    const afsRender = Metadata.getInstance<IterableComponentRender>('iterableCardsAF');
    tabsAfs.tabs = afsRender.datasource.data.map((row) => new Tab({
      name: `${row.KEY}`,
      tabTitle: `${this.utilities.getValueWithMaskProduto(row.CDPRODUTO)} - ${row.NMPRODUTO}`,
      children: [
        {
          allowDuplicate: true,
          name: `tagTab-${row.KEY}`,
          component: 'ZdTag',
          tag: 'div',
          isVisible: false,
        },
        {
          name: `formDetailAf-${row.CDPRODUTO}`,
          component: 'ZdForm',
          children: [
            {
              allowDuplicate: true,
              name: 'CDPRODUTO',
              component: 'ZdText',
              readonly: true,
              text: `${I18n.translate('Produto: ')} ${this.utilities.getValueWithMaskProduto(row.CDPRODUTO)} ${' | '}
              ${row.NMPRODUTO}`,
              grid: {
                cols: 12,
              },
            },
            {
              allowDuplicate: true,
              name: 'SGUNIDADE',
              component: 'ZdText',
              readonly: true,
              text: `${I18n.translate('Unidade: ')} ${row.SGUNIDADE}`,
              grid: {
                cols: 12,
              },
            },
            {
              allowDuplicate: true,
              name: 'QTPROGENTAF',
              component: 'ZdText',
              readonly: true,
              text: `${I18n.translate('Qtde')}: ${this.utilities.getMaskNumber(row.QTPROGENTAF || '', mask)}`,
              grid: {
                cols: 12,
              },
            },
            {
              allowDuplicate: true,
              name: 'NMMARCA',
              component: 'ZdText',
              readonly: true,
              text: `${I18n.translate('Marca')}: ${row.NMMARCA}`,
              grid: {
                cols: 12,
              },
            },
            {
              allowDuplicate: true,
              name: 'DTPROGENTAF',
              component: 'ZdText',
              readonly: true,
              text: `${I18n.translate('Dt Entrega')}: ${this.utilities.formatDate(row.DTPROGENTAF)}`,
              grid: {
                cols: 12,
              },
            },
            {
              allowDuplicate: true,
              name: 'VRPREUNIAF',
              component: 'ZdText',
              readonly: true,
              text: `${I18n.translate('Valor Unitário')}: ${this.utilities.getMaskNumber(row.VRPREUNIAF || '', mask)}`,
              grid: {
                cols: 12,
              },
            },
            {
              allowDuplicate: true,
              name: 'VRTOTALITEM',
              component: 'ZdText',
              readonly: true,
              text: `${I18n.translate('Valor Total')}: ${this.utilities.getMaskNumber(row.VRTOTALITEM || '', mask)}`,
              grid: {
                cols: 12,
              },
            },
            {
              allowDuplicate: true,
              name: 'NRAUTOFORN',
              component: 'ZdText',
              readonly: true,
              text: `${I18n.translate('AF')}: ${row.NRAUTOFORN}`,
              grid: {
                cols: 12,
              },
            },
            {
              allowDuplicate: true,
              name: 'CDFILLANC',
              component: 'ZdText',
              readonly: true,
              text: `${I18n.translate('Unidade Lançamento')}: ${row.CDFILLANC}`,
              grid: {
                cols: 12,
              },
            },
            {
              allowDuplicate: true,
              name: 'CDFILDEST',
              component: 'ZdText',
              readonly: true,
              text: `${I18n.translate('Unidade Destino: ')} ${row.CDFILDEST}`,
              grid: {
                cols: 12,
              },
            },
          ],
        },
      ],
    }));
    const index = component.name.split('-')[1];
    tabsAfs.activeTab = await tabsAfs.tabs.findIndex((tab) => tab.children[0].name === `tagTab-${index}`);
    LoadingService.hide();
  }

  public clickBackAFs() {
    this.showAfDetail = false;
    // this.showInventarioDetail = true;
    const afs = Metadata.getInstance<IterableComponentRender>('iterableCardsAF');
    afs.reload();
  }

  public clickFilterAF() {
    const bottomSheet = Metadata.getInstance<BottomSheet>('filterSheet');
    const filial = Metadata.getInstance<Select>('CDFILIALLOGADA');
    const hasImportedData = localStorage.getItem('hasImportedData');
    if (filial.value && hasImportedData === 'true') {
      this.utilities.fillDatesFilter(bottomSheet);
      bottomSheet.isVisible = true;
      bottomSheet.isOpen = true;
    }
  }

  public ordenaAF({ component }: IEventParam<Button>) {
    const { name } = component;
    const af = Metadata.getInstance<IterableComponentRender>('iterableCardsAF');
    const ds = af.datasource;
    const order = ds.getOrderByColumn(name);
    if (order) {
      if (order === 'desc') ds.addOrder(name, 'asc');
      else ds.addOrder(name, 'desc');
    } else ds.addOrder(name, 'desc');
  }

  public openFilter() {
    if (!localStorage.getItem('DATA_INICIAL') && !localStorage.getItem('DATA_FINAL')) {
      const btn = Metadata.getInstance<Button>('filterAFButton');
      btn.click();
    } else if (localStorage.getItem('CDFILIALLOGADA') !== 'null') {
      const dtIni = localStorage.getItem('DATA_INICIAL') || '';
      const dtFin = localStorage.getItem('DATA_FINAL') || '';
      this.loadAF(dtIni, dtFin, '');
    }
  }

  public clickCloseFilterBtn() {
    const bottomSheet = Metadata.getInstance<BottomSheet>('filterSheet');
    bottomSheet.isVisible = false;
    bottomSheet.isOpen = false;
  }

  public setPeriodoAF({ component }: IEventParam<Button>) {
    const form = component.parent as Form;
    if (form.validate()) {
      if (form.value.PERIODO.length === 2) {
        let format = 'YYYY-MM-DD';
        if (form.value.PERIODO[1].charAt(2) === '/') format = 'DD/MM/YYYY';
        const dtini = moment(form.value.PERIODO[0], format);
        const dtfin = moment(form.value.PERIODO[1], format);
        if (dtini.isValid() && dtfin.isValid()) {
          if (dtini >= moment(localStorage.getItem('DATA_INICIAL'), 'DD/MM/YYYY')
            && dtfin <= moment(localStorage.getItem('DATA_FINAL'), 'DD/MM/YYYY')) {
            this.loadAF(dtini.format('DD/MM/YYYY'), dtfin.format('DD/MM/YYYY'), form.value.search);
            const bottomSheet = Metadata.getInstance<BottomSheet>('filterSheet');
            bottomSheet.isVisible = false;
            bottomSheet.isOpen = false;
          } else this.utilities.showAlert('errorPeriodFilter', 'DATE_NOT_IN_PERIOD', 'error');
        } else this.utilities.showAlert('errorDateFilter', 'INVALID_DATE', 'error');
      } else this.utilities.showAlert('errorFilter', 'FILTER_2_DATES', 'error');
    }
  }

  public loadAF(inicio: string, fim: string, search: string) {
    this.periodo.DTINICIAL = inicio;
    this.periodo.DTFINAL = fim;
    const af = Metadata.getInstance<IterableComponentRender>('iterableCardsAF');
    af.datasource.search = search;
    // af.datasource.order = [];
    af.reload();
  }

  public checkAccess() {
    if (localStorage.getItem('hasImportedData') === 'true') {
      const filial = Metadata.getInstance<Select>('CDFILIALLOGADA');
      if (!filial.value) {
        this.appController.unidadeLogadaIsValid();
      }
    } else {
      DialogService.show({
        name: 'needImport',
        type: 'error',
        title: I18n.translate('scm::ERROR'),
        text: I18n.translate('scm::NEED_IMPORT'),
        buttons: [
          {
            name: 'ok',
            component: 'ZdButton',
            label: I18n.translate('scm::OK'),
            events: {
              click: () => {
                DialogService.hide();
                Router.push('/importacao');
              },
            },
          },
        ],
      });
    }
  }

  public setFormattedDate({ component }: IEventParam<Text>) {
    const value = component.name.split('-')[1] || '';
    component.text = `${I18n.translate('Data de Entrega')}: ${this.utilities.formatDate(value)}`;
    // "text": "Data de Entrega : <<row.DTPROGENTAF>>"
  }

  public setMarca({ component }: IEventParam<Text>) {
    const value = component.name.split('-')[2] || '';
    component.text = `${I18n.translate('Marca')}: ${value}`;
  }

  public setNrAF({ component }: IEventParam<Text>) {
    const value = component.name.split('-')[2] || '';
    component.text = `${I18n.translate('AF')}: ${value}`;
  }
}
