import {
  Loader, IEventParam, Metadata, dayjs as moment, I18n, IDictionary, DatasourceFactory, Config, Router, Http,
} from '@zeedhi/core';
import {
  Card, IterableComponentRender, Tabs, Tab, Increment, Button, LoadingService, Form, DialogService, Text, Select,
} from '@zeedhi/common';
import { BottomSheet } from '@zeedhi/zd-bottom-sheet';
import { getStoredData } from '@zeedhi/zeedhi-miragejs';
import { ScmMask } from '@/config';
import { HttpCRUDTranslator } from '@/http-crud-translator/http-crud-translator';

export class InventariosController {
  private utilities = Loader.getInstance('Utilities');

  private appController = Loader.getInstance('AppController');

  public showInventarioDetail = false;

  public showEditProdutosInventario = false;

  public showAddInventario = false;

  public showAddProduto = false;

  public inventarioDescription = '';

  public inventarioSelected = '';

  public notPossibleDates: string[] = [];

  private periodo = {
    DTINICIAL: '',
    DTFINAL: '',
  };

  public openAddInventario() {
    const form = Metadata.getInstance<Form>('addInventarioForm');
    form.clearForm();
    this.showAddInventario = true;
    this.setNotPossibleDates();
  }

  get isVisibleInventariosList() {
    return !this.showInventarioDetail && !this.showEditProdutosInventario && !this.showAddInventario && !this.showAddProduto;
  }

  public get setFilterInventarios() {
    if (localStorage.getItem('hasImportedData') === 'true') {
      return {
        CDFILIAL: localStorage.getItem('CDFILIALLOGADA') || '',
        DTINVENTRETI: this.utilities.getAllDatesInRange(this.periodo.DTINICIAL, this.periodo.DTFINAL),
      };
    }
    return {
      CDFILIAL: localStorage.getItem('CDFILIALLOGADA') || '',
      DTINICIAL: this.periodo.DTINICIAL || '',
      DTFINAL: this.periodo.DTFINAL || '',
    };
  }

  public get setFilterProdutosInventario() {
    return {
      CDFILIAL: localStorage.getItem('CDFILIALLOGADA') || '',
      NRINVENTRETI: this.inventarioSelected || '',
    };
  }

  public clickBackDetailInventario() {
    this.showInventarioDetail = false;
    this.inventarioSelected = '';
  }

  public clickBackEditProdutos() {
    this.showEditProdutosInventario = false;
    this.showInventarioDetail = true;
    const produtos = Metadata.getInstance<IterableComponentRender>('iterableCardsProdutosInventario');
    produtos.reload();
  }

  public clickBackAddInventario() {
    this.showAddInventario = false;
  }

  public clickBackAddProduto() {
    this.showAddProduto = false;
    this.showInventarioDetail = true;
    const produtos = Metadata.getInstance<IterableComponentRender>('iterableCardsProdutosInventario');
    produtos.reload();
  }

  public clickCardInventario({ component }: IEventParam<Card>) {
    this.showInventarioDetail = true;
    const split = component.name.split('_');
    const numero = split[1];
    const nome = split[2];
    const data = split[3];
    const dateFormat = this.utilities.formatDate(data);
    this.inventarioSelected = numero;
    this.inventarioDescription = `${dateFormat} - ${nome}`;
    const produtosRender = Metadata.getInstance<IterableComponentRender>('iterableCardsProdutosInventario');
    produtosRender.reload();
  }

  public async clickCardProdutoInventario({ component }: IEventParam<Card>) {
    LoadingService.show(' ');
    this.showEditProdutosInventario = true;
    this.showInventarioDetail = false;
    const tabsProdutos = Metadata.getInstance<Tabs>('tabsProdutos');
    const produtosRender = Metadata.getInstance<IterableComponentRender>('iterableCardsProdutosInventario');
    if (tabsProdutos.tabs.length > 0) {
      // destrói os elementos das tabs que vão ser substituídas para não acontecer conflito de elementos
      await Promise.all(tabsProdutos.tabs.map(async (tab: IDictionary) => {
        await tab.children.map(async (child: IDictionary) => {
          await tab.removeChild(child.name);
        });
      })).then(() => {
        tabsProdutos.tabs = [];
        tabsProdutos.tabs = produtosRender.datasource.data.map((row) => this.newTabs(row));
      });
    } else tabsProdutos.tabs = produtosRender.datasource.data.map((row) => this.newTabs(row));
    const index = component.name.split('-')[1];
    tabsProdutos.activeTab = await tabsProdutos.tabs.findIndex((tab) => tab.children[0].name === `tagTab-${index}`);
    LoadingService.hide();
  }

  private newTabs(row: IDictionary) {
    return new Tab({
      name: `${row.CDPRODUTO}`,
      tabTitle: `${this.utilities.getValueWithMaskProduto(row.CDPRODUTO)} - ${row.NMPRODUTO}`,
      children: [
        {
          allowDuplicate: true,
          name: `tagTab-${row.NRINVENTRETI}_${row.CDPRODUTO}`,
          component: 'ZdTag',
          tag: 'div',
          children: [
            {
              name: `formEditProduct-${row.NRINVENTRETI}_${row.CDPRODUTO}`,
              component: 'ZdForm',
              children: [
                {
                  allowDuplicate: true,
                  name: 'CDARVPROD',
                  component: 'ZdText',
                  // label: 'Código',
                  // value: `${row.CDARVPROD}`,
                  // mask: '{{Utilities.getMaskProduto}}',
                  text: `${I18n.translate('Produto: ')} ${this.utilities.getValueWithMaskProduto(row.CDARVPROD)} ${' | '}
                  ${row.NMPRODUTO}`,
                  grid: {
                    cols: 12,
                  },
                },
                {
                  allowDuplicate: true,
                  name: 'CDPRODUTO',
                  isVisible: false,
                  component: 'ZdTextInput',
                  readonly: true,
                  label: 'Código',
                  value: `${row.CDPRODUTO}`,
                  mask: '{{Utilities.getMaskProduto}}',
                  grid: {
                    cols: 12,
                  },
                },
                {
                  allowDuplicate: true,
                  // name: `nome-${row.NRINVENTRETI}_${row.CDPRODUTO}`,
                  name: 'NMPRODUTO',
                  component: 'ZdTextInput',
                  isVisible: false,
                  readonly: true,
                  label: 'Nome',
                  value: `${row.NMPRODUTO}`,
                  grid: {
                    cols: 12,
                  },
                },
                {
                  allowDuplicate: true,
                  // name: `quantidade-${row.NRINVENTRETI}_${row.CDPRODUTO}`,
                  name: 'QTITINVENTRETI',
                  component: 'ZdIncrement',
                  label: 'Quantidade',
                  clearable: false,
                  step: 1,
                  events: {
                    change: '{{InventariosController.changeQuantProduct}}',
                  },
                  validations: {
                    required: {
                      message: ' ',
                    },
                    min: {
                      limit: 0.1,
                    },
                  },
                  value: row.QTITINVENTRETI,
                  grid: {
                    cols: 12,
                  },
                },
                {
                  allowDuplicate: true,
                  name: 'NRITINVENTRETI',
                  component: 'ZdTextInput',
                  isVisible: false,
                  value: `${row.NRITINVENTRETI}`,
                },
              ],
            },
            {
              name: `salvarEditProdutoBtn-${row.NRINVENTRETI}_${row.CDPRODUTO}_${row.KEY}`,
              component: 'ZdButton',
              label: 'Salvar',
              block: true,
              fixed: true,
              bottom: true,
              disabled: true,
              ligth: false,
              dark: true,
              events: {
                click: '{{InventariosController.salvarEditProduto}}',
              },
              cssStyle: {
                right: '0px',
              },
            },
          ],
        },
      ],
    });
  }

  public setNotPossibleDates() {
    const inventarios = Metadata.getInstance<IterableComponentRender>('iterableCardsInventarios');
    const cloneConfig = inventarios.datasource.clone();
    const cloneDs = DatasourceFactory.factory(cloneConfig);
    cloneDs.loadAll = true;
    cloneDs.events = {};
    cloneDs.filter = {
      CDFILIAL: localStorage.getItem('CDFILIALLOGADA') || '',
      DTINICIAL: localStorage.getItem('DATA_INICIAL') ? localStorage.getItem('DATA_INICIAL') : this.periodo?.DTINICIAL || '',
      DTFINAL: localStorage.getItem('DATA_FINAL') ? localStorage.getItem('DATA_FINAL') : this.periodo?.DTFINAL || '',
    };
    LoadingService.show(' ');
    cloneDs.get().then((result: IDictionary) => {
      this.notPossibleDates = result.map((row: IDictionary) => {
        const dateRow = moment(row.DTINVENTRETI, 'DD/MM/YYYY');
        return dateRow.format(Config.dateFormat);
      });
    }).finally(() => {
      LoadingService.hide();
    });
  }

  public allowedDatesAddInventario(date: string) {
    let valid = true;
    const dateVerify = moment(date, Config.dateFormat);
    const inicio = localStorage.getItem('DATA_INICIAL')
      ? localStorage.getItem('DATA_INICIAL') : this.periodo?.DTINICIAL || '';
    const final = localStorage.getItem('DATA_FINAL') ? localStorage.getItem('DATA_FINAL') : this.periodo?.DTFINAL || '';
    const dtInicial = moment(inicio, 'DD/MM/YYYY');
    const dtFinal = moment(final, 'DD/MM/YYYY');
    const inventarios = Loader.getInstance('InventariosController');
    if (inventarios.notPossibleDates.includes(dateVerify.format(Config.dateFormat))) valid = false;
    if (dateVerify > dtFinal) valid = false;
    if (dateVerify < dtInicial) valid = false;
    return valid;
  }

  public editProximoCodigoInventario(usedCode: string) {
    const uniqueKey = 'CDFILIAL';
    const oldCode: number = +usedCode;
    const newCode = oldCode + 1;
    const data = {
      CDFILIAL: localStorage.getItem('CDFILIALLOGADA') || '',
      PROXIMOCODIGO: `${`${newCode}`.padStart(10, '0')}`,
    };
    Http.put(`proximoCodigoInventario/${data[uniqueKey]}`, data);
  }

  public async salvarInventario() {
    const form = Metadata.getInstance<Form>('addInventarioForm');
    if (form.validate()) {
      const { value } = form;
      const data = moment(value.DTINVENTRETI, Config.dateFormat);
      if (data.isValid()) {
        value.CDFILIAL = localStorage.getItem('CDFILIALLOGADA') || '';
        value.DTINVENTRETI = data.format('DD/MM/YYYY');
        LoadingService.show(' ');
        if (localStorage.getItem('hasImportedData') === 'true') {
          const ds = DatasourceFactory.factory({
            uniqueKey: 'CDFILIAL',
            type: 'rest',
            route: '/proximoCodigoInventario',
            filter: {
              CDFILIAL: localStorage.getItem('CDFILIALLOGADA') || '',
            },
          });
          await ds.get().then((result: IDictionary) => {
            value.NRINVENTRETI = result[0].PROXIMOCODIGO;
            this.editProximoCodigoInventario(result[0].PROXIMOCODIGO);
          });
        }
        HttpCRUDTranslator.post('/inventarios/save', { requestType: 'DataSet', dataset: [value] })
          .then((response: IDictionary) => {
            if (!response.data.error) {
              this.utilities.showAlert('successSave', 'SUCCESS_SAVE', 'green');
              const inventarios = Metadata.getInstance<IterableComponentRender>('iterableCardsInventarios');
              inventarios.datasource.setOrder(inventarios.datasource.order);
              inventarios.reload();
              // this.clickBackAddInventario();
              this.showAddInventario = false;
              this.showInventarioDetail = true;
              const dateFormat = this.utilities.formatDate(value.DTINVENTRETI);
              this.inventarioSelected = value.NRINVENTRETI;
              this.inventarioDescription = `${dateFormat} - ${value.DSINVENTRETI}`;
              const produtosRender = Metadata.getInstance<IterableComponentRender>('iterableCardsProdutosInventario');
              produtosRender.reload();
            } else {
              const mensagem = response.data.error;
              this.utilities.showAlert('errorSave', I18n.translate(mensagem), 'error');
            }
          }).catch((error: string) => {
            this.utilities.showAlert('alert', `${error}`, 'error');
          }).finally(() => {
            LoadingService.hide();
          });
      } else this.utilities.showAlert('errorDateForm', 'INVALID_DATE', 'error');
    }
  }

  public deleteInventario({ component }: IEventParam<Button>) {
    const nrInvent = component.name.split('-')[1];
    const key = component.name.split('-')[2];
    const dataset = [
      {
        CDFILIAL: localStorage.getItem('CDFILIALLOGADA') || '',
        NRINVENTRETI: nrInvent,
        // KEY: `${localStorage.getItem('CDFILIALLOGADA')}${nrInvent}`,
        KEY: key,
      },
    ];
    LoadingService.show(' ');
    HttpCRUDTranslator.post('/inventarios/delete', { requestType: 'DataSet', dataset }, {})
      .then((response: IDictionary) => {
        if (!response.data.error) {
          // this.utilities.showAlert('successSave', 'SUCCESS_SAVE', 'green', false);
          const inventarios = Metadata.getInstance<IterableComponentRender>('iterableCardsInventarios');
          inventarios.reload();
        } else {
          const mensagem = response.data.error;
          this.utilities.showAlert('errorSave', I18n.translate(mensagem), 'error');
        }
      }).catch((error: string) => {
        this.utilities.showAlert('alert', `${error}`, 'error');
      }).finally(() => {
        DialogService.hide();
        LoadingService.hide();
      });
  }

  public changeQuantProduct({ component }: IEventParam<Increment>) {
    const size = component.parent?.parent?.childrenInstances?.length || 0;
    const btn = component.parent?.parent?.childrenInstances[size - 1] as Button;
    btn.disabled = false;
  }

  public adicionarProdutoInventario() {
    const form = Metadata.getInstance<Form>('addProdutoInventarioForm');
    form.clearForm();
    this.showInventarioDetail = false;
    this.showAddProduto = true;
  }

  public get setFilterProdutos() {
    return {
      CDFILIAL: [localStorage.getItem('CDFILIALLOGADA')] || [],
      CDGRUPPROD: localStorage.getItem('CDGRUPPROD') || 'T',
      TIPOPRODUTO: 'E',
    };
  }

  public clickDeleteInventario({ component, event }: IEventParam<Button>) {
    (event as IDictionary).stopPropagation();
    DialogService.show({
      name: 'deleteInventario',
      type: 'warning',
      title: I18n.translate('scm::ATTENTION'),
      text: I18n.translate('scm::CONFIRM_DELETE_INVENTORY'),
      buttons: [
        {
          name: 'yes',
          component: 'ZdButton',
          label: I18n.translate('scm::YES'),
          events: { click: () => this.deleteInventario({ component }) },
          outline: true,
        },
        {
          name: 'no',
          component: 'ZdButton',
          label: I18n.translate('scm::NO'),
          events: { click: () => DialogService.hide() },
        },
      ],
    });
  }

  public clickDeleteProdutoInventario({ component, event }: IEventParam<Button>) {
    (event as IDictionary).stopPropagation();
    DialogService.show({
      name: 'deleteProdutoInventario',
      type: 'warning',
      title: I18n.translate('scm::ATTENTION'),
      text: I18n.translate('scm::CONFIRM_DELETE_REGISTER'),
      buttons: [
        {
          name: 'yes',
          component: 'ZdButton',
          label: I18n.translate('scm::YES'),
          events: { click: () => this.deleteProdutoInventario({ component }) },
          outline: true,
        },
        {
          name: 'no',
          component: 'ZdButton',
          label: I18n.translate('scm::NO'),
          events: { click: () => DialogService.hide() },
        },
      ],
    });
  }

  public deleteProdutoInventario({ component }: IDictionary) {
    const nrItInvent = component.name.split('-')[1].split('_')[0];
    const cdroduto = component.name.split('-')[1].split('_')[1];
    const key = component.name.split('-')[1].split('_')[2];
    const dataset = [
      {
        CDFILIAL: localStorage.getItem('CDFILIALLOGADA') || '',
        NRINVENTRETI: this.inventarioSelected,
        NRITINVENTRETI: nrItInvent,
        CDPRODUTO: cdroduto,
        // KEY: `${localStorage.getItem('CDFILIALLOGADA')}${this.inventarioSelected}${nrItInvent}`,
        KEY: key,
      },
    ];
    LoadingService.show(' ');
    HttpCRUDTranslator.post(
      '/produtosInventario/delete',
      { requestType: 'DataSet', dataset },
      {},
    ).then((response: IDictionary) => {
      if (!response.data.error) {
        // this.utilities.showAlert('successSave', 'SUCCESS_SAVE', 'green', false);
        const produtos = Metadata.getInstance<IterableComponentRender>('iterableCardsProdutosInventario');
        produtos.reload();
      } else {
        const mensagem = response.data.error;
        this.utilities.showAlert('errorSave', I18n.translate(mensagem), 'error');
      }
    }).catch((error: string) => {
      this.utilities.showAlert('alert', `${error}`, 'error');
    }).finally(() => {
      DialogService.hide();
      LoadingService.hide();
    });
  }

  public salvarProdutoInventario() {
    const form = Metadata.getInstance<Form>('addProdutoInventarioForm');
    if (form.validate()) {
      const produtos = Metadata.getInstance<IterableComponentRender>('iterableCardsProdutosInventario');
      const dsConfig = produtos.datasource.clone();
      dsConfig.loadAll = true;
      dsConfig.order = ['NRITINVENTRETI.asc'];
      dsConfig.filter = {
        CDFILIAL: localStorage.getItem('CDFILIALLOGADA') || '',
        NRINVENTRETI: this.inventarioSelected || '',
      };
      const clonedDs = DatasourceFactory.factory(dsConfig);
      LoadingService.show(' ');
      clonedDs.get().then((results: IDictionary) => {
        if (results.length > 0) {
          const lastCode = results.at(-1).NRITINVENTRETI;
          const oldCode: number = +lastCode;
          const newCode = `${(oldCode + 1)}`.padStart(4, '0');
          const productExist = results.find((el: IDictionary) => el.CDPRODUTO === form.value.CDPRODUTO?.CDPRODUTO);
          if (productExist === undefined) {
            this.callSaveProdutoInventario(form, newCode);
          } else {
            this.utilities.showAlert('alert', 'PRODUCT_EXISTS', 'error');
          }
        } else this.callSaveProdutoInventario(form, '0001');
      }).finally(() => {
        LoadingService.hide();
      });
    }
  }

  public callSaveProdutoInventario(form: Form, nrItem: string) {
    const filial = localStorage.getItem('CDFILIALLOGADA') || '';
    const dataset = [
      {
        CDFILIAL: filial,
        NRINVENTRETI: this.inventarioSelected,
        new: true,
        CDPRODUTO: form.value.CDPRODUTO.CDPRODUTO,
        CDARVPROD: form.value.CDPRODUTO.CDARVPROD,
        NMPRODUTO: form.value.CDPRODUTO.NMPRODUTO,
        QTITINVENTRETI: form.value.QTITINVENTRETI,
        NRITINVENTRETI: nrItem,
        // KEY: `${filial}${this.inventarioSelected}${nrItem}`,
        // ...form.value,
      },
    ];
    LoadingService.show(' ');
    HttpCRUDTranslator.post('/produtosInventario/save', { requestType: 'DataSet', dataset })
      .then((response: IDictionary) => {
        if (!response.data.error) {
          this.utilities.showAlert('successSave', 'SUCCESS_SAVE', 'green');
          form.clearForm();
        } else {
          const mensagem = response.data.error;
          this.utilities.showAlert('errorSave', I18n.translate(mensagem), 'error');
        }
      }).catch((error: string) => {
        this.utilities.showAlert('alert', error, 'error');
      }).finally(() => {
        LoadingService.hide();
      });
  }

  public salvarEditProduto({ component }: IEventParam<Button>) {
    // const form = Metadata.getInstance<Form>('formEditProduct');
    const form = component.parent?.getChildInstance<Form>(component.parent?.children[0].name);
    if (form?.validate()) {
      const key = component.name.split('_')[2];
      const dataset = [
        {
          CDFILIAL: localStorage.getItem('CDFILIALLOGADA') || '',
          NRINVENTRETI: this.inventarioSelected,
          new: false,
          ...form?.value,
          // KEY: `${localStorage.getItem('CDFILIALLOGADA')}${this.inventarioSelected}${form?.value.NRITINVENTRETI}`,
          KEY: key,
        },
      ];
      LoadingService.show(' ');
      HttpCRUDTranslator.post('/produtosInventario/update', { requestType: 'DataSet', dataset })
        .then((response: IDictionary) => {
          if (!response.data.error) {
            this.utilities.showAlert('successSave', 'SUCCESS_SAVE', 'green');
            component.disabled = true;
            const tabsProdutos = Metadata.getInstance<Tabs>('tabsProdutos');
            tabsProdutos.nextTab();
          } else {
            const mensagem = response.data.error;
            this.utilities.showAlert('errorSave', I18n.translate(mensagem), 'error');
          }
        }).catch((error: string) => {
          this.utilities.showAlert('alert', error, 'error');
        }).finally(() => {
          LoadingService.hide();
        });
    }
  }

  public setValueQtde({ component }: IEventParam<Text>) {
    const value = component.name.split('-')[1].split('_')[2] || '';
    const mask = Config.masks.productMask as ScmMask;
    const maskNumber = this.utilities.getMaskNumber(value, mask);
    component.text = `${maskNumber}`;
  }

  public setFormattedDate({ component }: IEventParam<Text>) {
    const value = component.name.split('_')[1] || '';
    component.text = `${I18n.translate('Data')}: ${this.utilities.formatDate(value)}`;
  }

  public setPeriodoInventarios({ component }: IEventParam<Button>) {
    const form = component.parent as Form;
    if (form.validate()) {
      if (form.value.PERIODO.length === 2) {
        let format = 'YYYY-MM-DD';
        if (form.value.PERIODO[1].charAt(2) === '/') format = 'DD/MM/YYYY';
        const dtini = moment(form.value.PERIODO[0], format);
        const dtfin = moment(form.value.PERIODO[1], format);
        if (dtini.isValid() && dtfin.isValid()) {
          if (dtini >= moment(localStorage.getItem('DATA_INICIAL'), 'DD/MM/YYYY')
            && dtfin <= moment(localStorage.getItem('DATA_FINAL'), 'DD/MM/YYYY')) {
            this.loadInventarios(dtini.format('DD/MM/YYYY'), dtfin.format('DD/MM/YYYY'), form.value.search);
            const bottomSheet = Metadata.getInstance<BottomSheet>('filterSheet');
            bottomSheet.isVisible = false;
            bottomSheet.isOpen = false;
          } else this.utilities.showAlert('errorPeriodFilter', 'DATE_NOT_IN_PERIOD', 'error');
        } else this.utilities.showAlert('errorDateFilter', 'INVALID_DATE', 'error');
      } else this.utilities.showAlert('errorFilter', 'FILTER_2_DATES', 'error');
    }
  }

  public loadInventarios(inicio: string, fim: string, search: string) {
    this.periodo.DTINICIAL = inicio;
    this.periodo.DTFINAL = fim;
    const inventarios = Metadata.getInstance<IterableComponentRender>('iterableCardsInventarios');
    // inventarios.datasource.setSearch(form.value.search);
    inventarios.datasource.search = search;
    // inventarios.datasource.clearOrder();
    // inventarios.datasource.order = [];
    inventarios.reload();
  }

  public clickFilterInventarios() {
    const bottomSheet = Metadata.getInstance<BottomSheet>('filterSheet');
    const filial = Metadata.getInstance<Select>('CDFILIALLOGADA');
    const hasImportedData = localStorage.getItem('hasImportedData');
    if (filial.value && filial.value.CDFILCENTPME && filial.value.CDFILIAL !== filial.value.CDFILCENTPME
      && hasImportedData === 'true') {
      this.utilities.fillDatesFilter(bottomSheet);
      bottomSheet.isVisible = true;
      bottomSheet.isOpen = true;
    }
  }

  public clickCloseFilterBtn() {
    const bottomSheet = Metadata.getInstance<BottomSheet>('filterSheet');
    bottomSheet.isVisible = false;
    bottomSheet.isOpen = false;
  }

  public openFilter() {
    if (!localStorage.getItem('DATA_INICIAL') && !localStorage.getItem('DATA_FINAL')) {
      const btn = Metadata.getInstance<Button>('filterInventariosButton');
      btn.click();
    } else if (localStorage.getItem('CDFILIALLOGADA') !== 'null') {
      const dtIni = localStorage.getItem('DATA_INICIAL') || '';
      const dtFin = localStorage.getItem('DATA_FINAL') || '';
      this.loadInventarios(dtIni, dtFin, '');
    }
  }

  public closeProdutosFilterBtn() {
    const bottomSheet = Metadata.getInstance<BottomSheet>('produtosfilterSheet');
    bottomSheet.isVisible = false;
    bottomSheet.isOpen = false;
  }

  public clickFilterProdutos() {
    const bottomSheet = Metadata.getInstance<BottomSheet>('produtosfilterSheet');
    bottomSheet.isVisible = true;
    bottomSheet.isOpen = true;
  }

  public setSearchProdutosFilter({ component }: IEventParam<Button>) {
    const form = component.parent as Form;
    if (form.validate()) {
      const produtos = Metadata.getInstance<IterableComponentRender>('iterableCardsProdutosInventario');
      produtos.datasource.search = form.value.searchProdutos;
      // produtos.datasource.order = [];
      produtos.reload();
      const bottomSheet = Metadata.getInstance<BottomSheet>('produtosfilterSheet');
      bottomSheet.isVisible = false;
      bottomSheet.isOpen = false;
    }
  }

  public confirmCopyProducts() {
    DialogService.show({
      name: 'confirmCopyProducts',
      type: 'info',
      title: I18n.translate('scm::COPY_LAST_INVENTORY'),
      text: I18n.translate('scm::COPY_PRODUCTS_INVENTORY'),
      buttons: [
        {
          name: 'yes',
          component: 'ZdButton',
          label: I18n.translate('scm::YES'),
          events: {
            click: () => {
              this.copiarUltimoInvertario();
            },
          },
          outline: true,
        },
        {
          name: 'no',
          component: 'ZdButton',
          label: I18n.translate('scm::NO'),
          events: { click: () => DialogService.hide() },
        },
      ],
    });
  }

  public copiarUltimoInvertario() {
    // const ds = DatasourceFactory.factory({
    //   uniqueKey: 'NRINVENTRETI',
    //   type: 'rest',
    //   route: '/ultimoInventario',
    //   filter: {
    //     CDFILIAL: localStorage.getItem('CDFILIALLOGADA') || '',
    //   },
    // });
    DialogService.hide();
    LoadingService.show(' ');
    // ds.get().then((result: IDictionary) => {
    getStoredData('ultimoInventario').then((data) => {
      const filial = localStorage.getItem('CDFILIALLOGADA') || '';
      const result = data?.filter((el: IDictionary) => el.CDFILIAL === filial) || [];
      if (result.length > 0) {
        const produtos = Metadata.getInstance<IterableComponentRender>('iterableCardsProdutosInventario');
        const dsConfig = produtos.datasource.clone();
        dsConfig.loadAll = true;
        dsConfig.filter = {
          CDFILIAL: localStorage.getItem('CDFILIALLOGADA') || '',
          NRINVENTRETI: this.inventarioSelected || '',
        };
        const clonedDs = DatasourceFactory.factory(dsConfig);
        clonedDs.get().then((products: IDictionary) => {
          if (products.length > 0) {
            HttpCRUDTranslator.post('/produtosInventario/delete', { requestType: 'DataSet', dataset: products }, {})
              .then((response: IDictionary) => {
                if (!response.data.error) {
                  this.salvaProdutosUltimoInventario(result);
                } else {
                  const mensagem = response.data?.error;
                  this.utilities.showAlert('errorSave', I18n.translate(mensagem), 'error');
                }
              }).catch((error: string) => {
                this.utilities.showAlert('alert', `${error}`, 'error');
                LoadingService.hide();
              });
          } else {
            this.salvaProdutosUltimoInventario(result);
          }
        });
      } else {
        LoadingService.hide();
        DialogService.show({
          name: 'errorCopyInventory',
          type: 'warning',
          title: I18n.translate('scm::ATTENTION'),
          text: I18n.translate('scm::NO_INVENTORY_REGISTRED'),
          buttons: [
            {
              name: 'ok',
              component: 'ZdButton',
              label: I18n.translate('scm::OK'),
              events: { click: () => DialogService.hide() },
            },
          ],
        });
      }
    });
  }

  public async salvaProdutosUltimoInventario(products: IDictionary) {
    await products.forEach((product: IDictionary) => {
      product.QTITINVENTRETI = 0;
      product.NRINVENTRETI = this.inventarioSelected || '';
      product.new = true;
      // product.KEY = `${product.CDFILIAL}${this.inventarioSelected}${product.NRITINVENTRETI}`;
      // KEY: `${filial}${this.inventarioSelected}${nrItem}`,
    });
    HttpCRUDTranslator.post('/produtosInventario/save', { requestType: 'DataSet', dataset: products })
      .then((response: IDictionary) => {
        if (response.length > 0) {
          let error = '';
          response.forEach((el: IDictionary) => {
            if (el.data.error) error = error.concat(el.data.error);
          });
          if (error !== '') throw new Error(error);
          else {
            const produtos = Metadata.getInstance<IterableComponentRender>('iterableCardsProdutosInventario');
            produtos.reload();
          }
        } else if (response.data && !response.data.error) {
          const produtos = Metadata.getInstance<IterableComponentRender>('iterableCardsProdutosInventario');
          produtos.reload();
        } else {
          const mensagem = response.data.error;
          this.utilities.showAlert('errorSave', I18n.translate(mensagem), 'error');
        }
      }).catch((error: string) => {
        this.utilities.showAlert('alert', `${error}`, 'error');
      }).finally(() => {
        LoadingService.hide();
      });
  }

  public checkAccess() {
    if (localStorage.getItem('hasImportedData') === 'true') {
      const filial = Metadata.getInstance<Select>('CDFILIALLOGADA');
      if (filial.value) {
        let text = '';
        if (!filial.value.CDFILCENTPME) {
          text = 'scm::BLOCK_ACCESS_INVENTORY_1';
          DialogService.show({
            name: 'blockAccessInventory',
            type: 'error',
            title: I18n.translate('scm::ATTENTION'),
            text: I18n.translate(text),
            buttons: [
              {
                name: 'ok',
                component: 'ZdButton',
                label: I18n.translate('scm::OK'),
                events: {
                  click: () => {
                    DialogService.hide();
                    Router.push('/');
                  },
                },
              },
            ],
          });
        } else if (filial.value.CDFILIAL === filial.value.CDFILCENTPME) {
          text = 'scm::BLOCK_ACCESS_INVENTORY';
          DialogService.show({
            name: 'blockAccessInventory',
            type: 'error',
            title: I18n.translate('scm::ATTENTION'),
            text: I18n.translate(text),
            buttons: [
              {
                name: 'ok',
                component: 'ZdButton',
                label: I18n.translate('scm::OK'),
                events: {
                  click: () => {
                    DialogService.hide();
                    Router.push('/');
                  },
                },
              },
            ],
          });
        } else {
          this.checkStock();
        }
      } else {
        this.appController.unidadeLogadaIsValid();
      }
    } else {
      DialogService.show({
        name: 'needImport',
        type: 'error',
        title: I18n.translate('scm::ERROR'),
        text: I18n.translate('scm::NEED_IMPORT'),
        buttons: [
          {
            name: 'ok',
            component: 'ZdButton',
            label: I18n.translate('scm::OK'),
            events: {
              click: () => {
                DialogService.hide();
                Router.push('/importacao');
              },
            },
          },
        ],
      });
    }
  }

  public checkStock() {
    const ds = DatasourceFactory.factory({
      uniqueKey: 'CDFILIAL',
      type: 'rest',
      route: '/parametros',
      filter: {
        CDFILIAL: localStorage.getItem('CDFILIALLOGADA') || '',
      },
    });
    LoadingService.show(' ');
    ds.get().then((result: IDictionary) => {
      let text = '';
      if (result.length && result[0].IDUTILALMOX === 'S') {
        text = 'Esta unidade possui controle de estoque por almoxarifado, impossível lançar inventários!';
      } else if (result.length && result[0].IDUTILLOTE === 'S') {
        text = 'Esta unidade possui controle de estoque por lote, impossível lançar inventários!';
      }
      if (text !== '') {
        DialogService.show({
          name: 'blockAccessInventoryStock',
          type: 'error',
          title: I18n.translate('scm::ATTENTION'),
          text: I18n.translate(text),
          buttons: [
            {
              name: 'ok',
              component: 'ZdButton',
              label: I18n.translate('scm::OK'),
              events: {
                click: () => {
                  DialogService.hide();
                  Router.push('/');
                },
              },
            },
          ],
        });
      }
    }).finally(() => {
      LoadingService.hide();
    });
  }

  public get checkDisabledAddButton() {
    const filial = Metadata.getInstance<Select>('CDFILIALLOGADA');
    return !filial.value || this.periodo?.DTINICIAL === '' || this.periodo.DTFINAL === '';
  }
}
