import {
  IDictionary, IHttpResponse, Interceptor, I18n,
} from '@zeedhi/core';

export interface RequestLog {
  method: 'put' | 'post' | 'delete';
  url: string;
  row: IDictionary;
  operation: string;
  timestamp: number;
  route: string;
}

export interface RequestErrorLog {
  method: string;
  url: string;
  row: string;
  operation: string;
  timestamp: number;
  error: string;
  seen: boolean;
}

const routesToLog = [
  'inventarios',
  'produtosInventario',
  'servicosCardapioEfetivos',
  'clientesDoServicoEfetivos',
  'produtosSolicitacao',
];

const registerLogger = () => {
  Interceptor.addResponseSuccess((response: IHttpResponse<{ data: IDictionary }>) => {
    // se estiver online não precisamos salvar no histórico
    // if (navigator.onLine) return response;

    // const { statusText } = response;
    const { method } = response.config;
    const url = response.config.url || '';
    const route = url.split('/')[0];
    // cria um log ao adicionar/modificar/deletar um recurso
    if ((method === 'post' || method === 'put' || method === 'delete') && routesToLog.includes(route)) {
      const timestamp = Date.now();
      let operation = '';
      switch (method) {
        case 'post':
          operation = 'create';
          break;
        case 'put':
          operation = 'update';
          break;
        case 'delete':
          operation = 'delete';
          break;
        default:
          break;
      }

      let row = response.data.data;
      if (method === 'put' || method === 'delete' || method === 'post') {
        row = JSON.parse(response.config.data);
      } else row = [];
      const newLog: RequestLog = {
        method, url, row, operation, timestamp, route,
      };
      const item = localStorage.getItem('request-log') || '[]';
      const logs: RequestLog[] = JSON.parse(item);

      logs.push(newLog);

      // salva o log na localstorage
      const stringified = JSON.stringify(logs);
      localStorage.setItem('request-log', stringified);
    }

    return response;
  });
};

const registerErrorLogger = (response: IDictionary) => {
  const item = localStorage.getItem('request-error-log') || '[]';
  const errorLogs: RequestErrorLog[] = JSON.parse(item);

  const statusText = response?.statusText || '';
  const { method } = response?.config;
  const url = response?.config?.url || '';
  const row = JSON.stringify(response?.data?.data) || '';
  const timestamp = Date.now();
  const operation = statusText?.slice(0, -1).toLowerCase();
  let { error } = response?.data || '';
  if (error?.length !== 0 && error?.error !== null) {
    const mensagem = error?.error || '';
    if (mensagem.indexOf('ERROR_CHANGE_QUANTITY_EXPORT') === 0) {
      const param = mensagem.split(':');
      error = I18n.translate('scm::ERROR_CHANGE_QUANTITY_EXPORT', { PARAM1: param[1] });
    } else if (mensagem.indexOf('ERROR_DELETE_PRODUCT_EXPORT') === 0) {
      const param = mensagem.split(':');
      error = I18n.translate('scm::ERROR_DELETE_PRODUCT_EXPORT', { PARAM1: param[1] });
    }
  }
  const newLog: RequestErrorLog = {
    method, url, row, operation, timestamp, error, seen: false,
  };
  errorLogs.push(newLog);

  // salva o log na localstorage
  const stringified = JSON.stringify(errorLogs);
  localStorage.setItem('request-error-log', stringified);
};

export { registerLogger, registerErrorLogger };
